@@ -19,6 +19,36 @@ def task_name_to_key(x):
 sort_names = partial(sorted, key=task_name_to_key)
 
 
+def print_help(tuples):
+    indent = 2
+    padding = 3
+    # Calculate column sizes: don't wrap flag specs, give what's left over
+    # to the descriptions.
+    flag_width = max(len(x[0]) for x in tuples)
+    desc_width = pty_size()[0] - flag_width - indent - padding - 1
+    wrapper = textwrap.TextWrapper(width=desc_width)
+    for flag_spec, help_str in tuples:
+        # Wrap descriptions/help text
+        help_chunks = wrapper.wrap(help_str)
+        # Print flag spec + padding
+        flag_padding = flag_width - len(flag_spec)
+        spec = ''.join((
+            indent * ' ',
+            flag_spec,
+            flag_padding * ' ',
+            padding * ' '
+        ))
+        # Print help text as needed
+        if help_chunks:
+            print(spec + help_chunks[0])
+            for chunk in help_chunks[1:]:
+                print((' ' * len(spec)) + chunk)
+        else:
+            print(spec)
+    print('')
+
+
+
 def parse_gracefully(parser, argv):
     """
     Run ``parser.parse_argv(argv)`` & gracefully handle ``ParseError``.
@@ -58,9 +88,8 @@ def parse(argv, collection=None):
         ),
         Argument(
             names=('help', 'h'),
-            kind=bool,
-            default=False,
-            help="Show this help message and exit."
+            optional=True,
+            help="Show core or per-task help and exit."
         ),
         Argument(
             names=('version', 'V'),
@@ -115,41 +144,15 @@ def parse(argv, collection=None):
         print("Invoke %s" % __version__)
         sys.exit(0)
 
-    # Core --help output
+    # Core (no value given) --help output
     # TODO: if this wants to display context sensitive help (e.g. a combo help
     # and available tasks listing; or core flags modified by plugins/task
     # modules) it will have to move farther down.
-    if args.help.value:
+    if args.help.value == True:
         print("Usage: inv[oke] [--core-opts] task1 [--task1-opts] ... taskN [--taskN-opts]")
         print("")
         print("Core options:")
-        indent = 2
-        padding = 3
-        # Calculate column sizes: don't wrap flag specs, give what's left over
-        # to the descriptions.
-        tuples = initial_context.help_tuples()
-        flag_width = max(len(x[0]) for x in tuples)
-        desc_width = pty_size()[0] - flag_width - indent - padding - 1
-        wrapper = textwrap.TextWrapper(width=desc_width)
-        for flag_spec, help_str in tuples:
-            # Wrap descriptions/help text
-            help_chunks = wrapper.wrap(help_str)
-            # Print flag spec + padding
-            flag_padding = flag_width - len(flag_spec)
-            spec = ''.join((
-                indent * ' ',
-                flag_spec,
-                flag_padding * ' ',
-                padding * ' '
-            ))
-            # Print help text as needed
-            if help_chunks:
-                print(spec + help_chunks[0])
-                for chunk in help_chunks[1:]:
-                    print((' ' * len(spec)) + chunk)
-            else:
-                print(spec)
-        print('')
+        print_help(initial_context.help_tuples())
         sys.exit(0)
 
     # Load collection (default or specified) and parse leftovers
@@ -162,6 +165,37 @@ def parse(argv, collection=None):
     debug("Parsing actual tasks against collection %r" % collection)
     tasks = parse_gracefully(parser, core.unparsed)
 
+    # Per-task help. Use the parser's contexts dict as that's the easiest way
+    # to obtain Context objects here - which are what help output needs.
+    name = args.help.value
+    if name in parser.contexts:
+        # Setup
+        ctx = parser.contexts[name]
+        tuples = ctx.help_tuples()
+        docstring = collection[name].__doc__
+        header = "Usage: inv[oke] [--core-opts] %s %%s[other tasks here ...]" % name
+        print(header % ("[--options] " if tuples else ""))
+        print("")
+        print("Docstring:")
+        if docstring:
+            # Really wish textwrap worked better for this.
+            doclines = docstring.lstrip().splitlines()
+            for line in doclines:
+                print("    " + textwrap.dedent(line))
+            # Print trailing blank line if docstring didn't end with one
+            if textwrap.dedent(doclines[-1]):
+                print("")
+        else:
+            print("    none")
+            print("")
+        print("Options:")
+        if tuples:
+            print_help(tuples)
+        else:
+            print("    none")
+            print("")
+        sys.exit(0)
+
     # Print discovered tasks if necessary
     if args.list.value:
         print("Available tasks:\n")
@@ -183,7 +217,6 @@ def parse(argv, collection=None):
 
 def derive_opts(args):
     run = {}
-    # FIXME: deal with dash to underscore
     if args['warn-only'].value:
         run['warn'] = True
     if args.pty.value:
@@ -206,7 +239,6 @@ def dispatch(argv):
         try:
             # TODO: allow swapping out of Executor subclasses based on core
             # config options
-            # FIXME: friggin dashes vs underscores
             results.append(executor.execute(
                 name=context.name,
                 kwargs=kwargs,
