@@ -74,7 +74,7 @@ Core options:
   -c STRING, --collection=STRING   Specify collection name to load. May be
                                    given >1 time.
   -e, --echo                       Echo executed commands before running.
-  -h, --help                       Show this help message and exit.
+  -h [STRING], --help[=STRING]     Show core or per-task help and exit.
   -H STRING, --hide=STRING         Set default value of run()'s 'hide' kwarg.
   -l, --list                       List available tasks.
   -p, --pty                        Use a pty when executing shell commands.
@@ -90,10 +90,59 @@ Core options:
         eq_(r1.stdout, expected)
         eq_(r2.stdout, expected)
 
+    @trap
+    def per_task_help_prints_help_for_task_only(self):
+        expected = """
+Usage: inv[oke] [--core-opts] punch [--options] [other tasks here ...]
+
+Docstring:
+    none
+
+Options:
+  -h STRING, --why=STRING   Motive
+  -w STRING, --who=STRING   Who to punch
+
+""".lstrip()
+        r1 = run("inv -c decorator -h punch", hide='out')
+        r2 = run("inv -c decorator --help punch", hide='out')
+        eq_(r1.stdout, expected)
+        eq_(r2.stdout, expected)
+
+    @trap
+    def per_task_help_works_for_unparameterized_tasks(self):
+        expected = """
+Usage: inv[oke] [--core-opts] biz [other tasks here ...]
+
+Docstring:
+    none
+
+Options:
+    none
+
+""".lstrip()
+        r = run("inv -c decorator -h biz", hide='out')
+        eq_(r.stdout, expected)
+
+    @trap
+    def per_task_help_displays_docstrings_if_given(self):
+        expected = """
+Usage: inv[oke] [--core-opts] foo [other tasks here ...]
+
+Docstring:
+    Foo the bar.
+    
+Options:
+    none
+
+""".lstrip()
+        r = run("inv -c decorator -h foo", hide='out')
+        eq_(r.stdout, expected)
+
     @trap
     def version_info(self):
         eq_(run("invoke -V").stdout, "Invoke %s\n" % invoke.__version__)
 
+
     class task_list:
         "--list"
 
